<?
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Config
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Config/Config.php';
require_once 'Zend/Config/Exception.php';

class Webtatic_Config_Php extends Webtatic_Config
{
    public function __construct($filename, $section, $allowModifications = false)
    {
        if (is_array($filename)) {
            $vars = $filename;
        } else {
            $vars = $this->getVars($filename);
        }
        if (null === $section) {
            parent::__construct($vars, $allowModifications);
        } elseif (is_array($section)) {
            $dataArray = array();
            foreach ($section as $sectionName) {
                if (!isset($vars[$sectionName])) {
                    throw new Zend_Config_Exception("Section '$sectionName' cannot be found in $filename");
                }
                $dataArray = array_merge($vars[$sectionName], $dataArray);

            }
            parent::__construct($dataArray, $allowModifications);
        } else {
            if (!isset($vars[$section])) {
                throw new Zend_Config_Exception("Section '$section' cannot be found in $filename");
            }
            $dataArray = $vars[$section];

            parent::__construct($dataArray, true);

            $sectionExtends = (array)$vars[$section]['extends'];
            while ($sectionExtends) {
                $extend = array_shift($sectionExtends);
                $config = new Webtatic_Config($vars[$extend],
                    $allowModifications);
                $config->mergeTo($this);
                if (isset($vars[$extend]['extends'])) {
                    $sectionExtends = array_merge($sectionExtends,
                        (array)$vars[$extend]['extends']);
                }
            }

            if (!$allowModifications) {
                $this->setReadOnly();
            }
        }

        $this->_loadedSection = $section;
    }
    
    protected function getVars($filename)
    {
        return require_once($filename);
    }
}
