<?php

/** Zend_Controller_Router_Route_Interface */
require_once 'Zend/Controller/Router/Route/Interface.php';
require_once 'Webtatic/Parser/Abstract.php';

class Webtatic_Controller_Router_Route implements Zend_Controller_Router_Route_Interface
{
    protected $_route;
    protected $_defaults;
    protected $_reqs;

    protected $_regex;
    protected $_keys;
    protected $_tree;

    protected $_values;

    public function getVersion()
    {
        return 2;
    }

    public function __construct($route, $defaults, $reqs)
    {
        $this->_route = $route;
        $this->_defaults = $defaults;
        $this->_reqs = $reqs;

        $this->_compile();
    }

    protected function _compile()
    {
        $parser = new Webtatic_Controller_Router_Route_Parser();
        $result = $parser->parse($this->_route, $this->_reqs);
        $this->_regex = $result['regex'];
        $this->_keys = $result['keys'];
        $this->_tree = $result['tree'];
    }

    /**
     * @param Zend_Controller_Request_Http $request Request to get the host from
     * @return array|false
     */
    public function match($request)
    {
        $path = ltrim($request->getPathInfo(), '/');

        if (preg_match($this->_regex, $path, $matches)) {
            $params = array();
            foreach ($this->_keys as $index => $key) {
                if (!in_array($matches[$index+1], array('', null), true)) {
                    $params[$key] = $matches[$index+1];
                }
            }
            $this->_values = $params;
            $params += $this->_defaults;
            return $params;
        }
        return false;
    }

    public function assemble($data = array(), $reset = false, $encode = false)
    {
        if (!$reset && $this->_values) {
            $data = $data + $this->_values;
        }
        $path = $this->_map($data, $this->_tree, false, $encode);

        return $path;
    }

    protected function _map($data, $tree, $optional = false, $encode = false)
    {
        $path = '';
        foreach ($tree->items as $key => $value) {
            if (is_int($key)) {
                if (is_object($value)) {
                    $path .= $this->_map($data, $value, true, $encode);
                } else {
                    $path .= $value;
                }
            } elseif ($optional && isset($data[$key])
                && $data[$key] != $this->_defaults[$key]) {
                $optional = false;
                $path .= $data[$key];
            } elseif (!isset($data[$key])) {
                $path .= $this->_defaults[$key];
            } else {
                $path .= $data[$key];
            }
        }
        if (!$optional) {
            return $path;
        } else {
            return '';
        }
    }

    /**
     * Instantiates route based on passed Zend_Config structure
     *
     * @param Zend_Config $config Configuration object
     */
    public static function getInstance(Zend_Config $config)
    {
        $defs = ($config->defaults instanceof Zend_Config)
            ? $config->defaults->toArray()
            : array();
        $reqs = ($config->reqs instanceof Zend_Config)
            ? $config->reqs->toArray()
            : array();
        return new self($config->route, $defs, $reqs);
    }
}

class Webtatic_Controller_Router_Route_Parser extends Webtatic_Parser_Abstract
{
    protected $_defaultState = 'standard';
    protected $_missCallback = 'path';

    protected $_patterns = array(
        'token' => array(
            array('optional' => '\{([^\}]+)\}'),
            array('parameter' => ':([a-zA-Z_]+)'),
            array('parameter' => ':\(([^\:)]+)\)'),
            array('path' => '[^\:\{]+'),
        ),
        'standard' => array(
            array('#token', '#standard'),
        ),
    );

    protected $_reqs;
    protected $_defaultReq;

    protected $_regex;
    protected $_keys;
    protected $_tree;
    protected $_node;

    public function parse($route, $reqs, $defaultReq = '[^/]+')
    {
        $this->_reqs = $reqs;
        $this->_defaultReq = $defaultReq;

        $this->_regex = '';
        $this->_keys = array();
        $this->_tree = (object) array('items' => array());

        $this->_node = $this->_tree;
        $this->_parse($route);

        return array(
            'regex' => '#^' . $this->_regex . '$#',
            'keys' => $this->_keys,
            'tree' => $this->_tree,
        );
    }

    public function formatOptional($match)
    {
        $node = $this->_node;

        $this->_node = (object) array('items' => array());
        $this->_regex .= '(?:';
        $this->_parse($match[1][0]);
        $this->_regex .= ')?';

        $node->items[] = $this->_node;
        $this->_node = $node;
    }

    public function formatParameter($match)
    {
        $this->_keys[] = $match[1][0];
        $req = isset($this->_reqs[$match[1][0]]) ? $this->_reqs[$match[1][0]] : $this->_defaultReq;
        $this->_regex .= '('.$req.')';
        $this->_node->items[$match[1][0]] = null;
    }

    public function formatPath($match)
    {
        $this->_regex .= preg_quote($match[0][0], '#');
        $this->_node->items[] = $match[0][0];
    }
}
