<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Db
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Db/Table/Behaviour/Interface.php';

abstract class Webtatic_Db_Table_Behaviour_Abstract
    implements Webtatic_Db_Table_Behaviour_Interface
{

    protected $_table;

    public function __construct($table, $options)
    {
        $this->setTable($table);
        $this->setOptions($options);
    }

    /**
     * @return Webtatic_Db_Table_Abstract
     */
    public function getTable()
    {
        return $this->_table;
    }

    public function setTable($value)
    {
        $this->_table = $value;

        return $this;
    }

    public function setOption($name, $value)
    {
        if (method_exists($this, 'set'.$name)) {
            return call_user_func(array($this, 'set'.$name), $value);
        }
    }

    public function setOptions($options)
    {
        foreach ($options as $name => $value) {
            $this->setOption($name, $value);
        }

        return $this;
    }

    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function insert($row)
    {
    }

    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function postInsert($row)
    {
    }

    /**
     * Allows pre-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function update($row)
    {
    }

    /**
     * Allows post-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function postUpdate($row)
    {
    }

    /**
     * Allows pre-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function delete($row)
    {
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @return void
     */
    public function postDelete($row)
    {
    }
}
