<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Db
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Db/Table/Behaviour/Abstract.php';

class Webtatic_Db_Table_Behaviour_Ordered
    extends Webtatic_Db_Table_Behaviour_Abstract
{

    protected $_key = null;
    protected $_orderColumn = 'position';


    public function getKey()
    {
        return $this->_key;
    }

    public function setKey($value)
    {
        $this->_key = (array) $value;

        return $this;
    }

    public function getOrderColumn()
    {
        return $this->_orderColumn;
    }

    public function setOrderColumn($value)
    {
        $this->_orderColumn = $value;

        return $this;
    }
    
    /**
     * Enter description here...
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @param bool $dirty
     */
    protected function _getKeyData($row, $useDirty = true)
    {
        return $row->getData($this->_key, $useDirty);
    }
    
    /**
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     * @param bool $dirty
     * @return Webtatic_Db_Table_Select $row
     */
    protected function _select($row, $useDirty = true)
    {
        $data = $this->_getKeyData($row, $useDirty);
        $adapter = $this->getTable()->getAdapter();
        $select = $this->getTable()->select();
        foreach ($data as $key => $value) {
            $select->where($adapter->quoteIdentifier($key) . ' = ?', $value);
        }

        return $select;
    }

    /**
     * Enter description here...
     *
     * @param Webtatic_Db_Table_Row_Abstract $row
     */
    public function insert($row)
    {
        $this->_reorder($row);
    }

    public function update($row)
    {
        $this->_reorder($row);
    }
    
    protected function _reorder($row)
    {
        $adapter = $this->getTable()->getAdapter();

        $orderColumn = $adapter->quoteIdentifier($this->_orderColumn);
                
        $data = $this->_getKeyData($row);
        $where = array();
        foreach ($data as $key => $value) {
            $where[] = $adapter->quoteIdentifier($key) . ' = ' . $adapter->quote($value);
        }
        
        if ($order = $row->getData($this->_orderColumn)) {
            if ($oldOrder = $row->getData($this->_orderColumn)) {
                if ($order > $oldOrder) {
                    $this->_shiftNodes($where, $oldOrder, -1, $order);
                } else if ($order < $oldOrder) {
                    $this->_shiftNodes($where, $order, 1, $oldOrder);
                }
            } else {
                $this->_shiftNodes($where, $order, 1);
            }
        } else {
            if ($oldOrder = $row->getData($this->_orderColumn)) {
                $this->_shiftNodes($where, $oldOrder, -1);
            }
            
            $rightOf = $this->_select($row)
                ->from($this->getTable(), new Zend_Db_Expr("MAX($orderColumn)"))
                ->fetchOne();

            $row->{$this->_orderColumn} = $rightOf+1;
        }
    }
    
    protected function _shiftNodes($where, $from, $count, $toExclusive = null)
    {
        $adapter = $this->getTable()->getAdapter();

        $orderColumn = $adapter->quoteIdentifier($this->_orderColumn);

        $updateWhere = array_merge($where, array(
            $adapter->quoteInto("$orderColumn >= ?", $from),
        ));

        if ($toExclusive) {
            $updateWhere[] = $adapter->quoteInto("$orderColumn < ?", $toExclusive);
        }

        $adapter->update($this->getTable()->info('name'), array(
            $this->_orderColumn => new Zend_Db_Expr($adapter->quoteInto("$orderColumn + ?", $count)),
        ), $updateWhere);
    }
}