<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Db
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Zend/Db/Table/Rowset/Abstract.php';

abstract class Webtatic_Db_Table_Rowset_Abstract
    extends Zend_Db_Table_Rowset_Abstract
{

    protected $_foundRows = null;

    /**
     * Constructor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        if (isset($config['foundRows'])) {
            $this->_foundRows = $config['foundRows'];
        }
        parent::__construct($config);
    }

    public function getFoundRows()
    {
        return $this->_foundRows;
    }
    
    public function current()
    {
        if ($this->valid() === false) {
            return null;
        }

        // do we already have a row object for this position?
        if (empty($this->_rows[$this->_pointer])) {
            $this->_rows[$this->_pointer] = $this->_getRow($this->_data[$this->_pointer]);
        }

        // return the row object
        return $this->_rows[$this->_pointer];
    }
    
    protected function _getRow($data)
    {
        return new $this->_rowClass(
                array(
                    'table'    => $this->_table,
                    'data'     => $this->_data[$this->_pointer],
                    'stored'   => $this->_stored,
                    'readOnly' => $this->_readOnly
                )
            );
    }
}