<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Db
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Zend/Db/Statement.php';

class Webtatic_Db_UnpreparedStatement extends Zend_Db_Statement
{
    /**
     * Mysqli result set
     *
     * @var mysqli_result
     */
    protected $_resultSet = false;
    protected $_field_count = 0;
    protected $_sql;
    
    public function closeCursor()
    {
        if ($stmt = $this->_resultSet) {
            $this->_resultSet->free();
            $this->_resultSet = false;
            return true;
        }
        return false;
    }

    public function columnCount()
    {
        return $this->_field_count;
    }

    public function errorCode()
    {
        return substr($this->_adapter->getConnection()->sqlstate, 0, 5);
    }

    public function errorInfo()
    {
        return array(
            substr($this->_adapter->getConnection()->sqlstate, 0, 5),
            $this->_adapter->getConnection()->errno,
            $this->_adapter->getConnection()->error,
        );
    }

    public function _prepare($sql)
    {
        if (is_object($sql)) {
            if (method_exists($sql, '__toString')) {
                $this->_sql = $sql->__toString();
            } else {
                throw new Exception('__toString does not exist', 0);    
            }
        } else {
            $this->_sql = $sql;
        }
    }

    protected function _parseParameters($sql)
    {
         
    }
    
    public function close()
    {
        $this->closeCursor();
    }
    
    public function getQuery(array $params = null)
    {
        $p = $this->_bindParam;
        array_splice($p, count($p), 0, $params);
        $sqlParams = split('\?', $this->_sql.' ');
        
        $sql = array_shift($sqlParams);
        
        foreach ($sqlParams as $part) {
            $v = array_shift($p);
            if (is_null($v)) {
                $sql .= 'NULL'.$part;
            } else {
                $sql .= $this->_adapter->quote($v).$part;
            }
        }
        return $sql;
    }
    
    public function _execute(array $params = null)
    {
        $sql = $this->getQuery($params);
        
        if (method_exists($this->_adapter, 'lastQuery')) {
            $this->_adapter->lastQuery($sql);
        }
        $ret = $this->_adapter->getConnection()->real_query($sql);
        
        $this->_field_count = $this->_adapter->getConnection()->field_count;

        if ($ret === false) {
            /**
             * @see Zend_Db_Statement_Mysqli_Exception
             */
            require_once 'Zend/Db/Statement/Mysqli/Exception.php';
            throw new Zend_Db_Statement_Mysqli_Exception("Mysqli statement execute error : " . $this->_adapter->getConnection()->error);
        }
        
        if ($this->_field_count > 0) {
            $this->_resultSet = $this->_adapter->getConnection()->store_result();
        }
    }

    /**
     * Fetches a row from the result set.
     *
     * @param int $style  OPTIONAL Fetch mode for this fetch operation.
     * @param int $cursor OPTIONAL Absolute, relative, or other.
     * @param int $offset OPTIONAL Number for absolute or relative cursors.
     * @return mixed Array, object, or scalar depending on fetch mode.
     * @throws Zend_Db_Statement_Mysqli_Exception
     */
    public function fetch($style = null, $cursor = null, $offset = null)
    {
        if ($style == null) {
            $style = $this->_fetchMode;
        }
        switch ($style) {
            case Zend_Db::FETCH_NUM:
                $mode = MYSQLI_NUM;
                break;
            case Zend_Db::FETCH_OBJ:
            case Zend_Db::FETCH_ASSOC:
                $mode = MYSQLI_ASSOC;
                break;
            case Zend_Db::FETCH_BOTH:
                $mode = MYSQLI_BOTH;
                break;
            default:
                /**
                 * @see Zend_Db_Statement_Mysqli_Exception
                 */
                require_once 'Zend/Db/Statement/Mysqli/Exception.php';
                throw new Zend_Db_Statement_Mysqli_Exception("Invalid fetch mode '$style' specified");
                break;
        }
        if (!is_object($this->_resultSet)) {
            throw new Exception('Mysqli resultset not valid', 0);
        }
        $retval = $this->_resultSet->fetch_array($mode);
        switch ($retval) {
            case null: // end of data
            case false: // error occurred
                $this->_resultSet = null;
                return $retval;
            default:
                // fallthrough
        }
        
        if ($style == Zend_Db::FETCH_OBJ) {
            $retval = (object) $retval;
        }
        return $retval;
    }

    public function nextRowset()
    {
        /**
         * @see Zend_Db_Statement_Mysqli_Exception
         */
        require_once 'Zend/Db/Statement/Mysqli/Exception.php';
        throw new Zend_Db_Statement_Mysqli_Exception(__FUNCTION__.'() is not implemented');
    }

    public function rowCount()
    {
        if (!$this->_adapter) {
            return false;
        }
        $mysqli = $this->_adapter->getConnection();
        return $mysqli->affected_rows;
    }

}
?>
