<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Filesystem
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

class Webtatic_Filesystem_DirectoryIterator implements Iterator
{
    
    const RECURSE = 'recurse';
    const FOLLOW_SYMLINKS = 'follow-symlinks';
    const MODE = 'mode';
    const CALLBACK = 'callback';
    const SORT = 'sort';
    
    const MODE_DIRS = 1;
    const MODE_NONDIRS = 2;
    const MODE_BOTH = 3;
    
    const SORT_BREADTH_FIRST = 1;
    const SORT_DEPTH_FIRST = 2;
    const SORT_BOTH = 3;
    
    protected $_subdirs;
    protected $_dirs;
    protected $_dir = null;
    protected $_handle;
    protected $_file;
    
    protected $_index;
    
    protected $_basePath;
    protected $_options = array(
        self::RECURSE => false,
        self::FOLLOW_SYMLINKS => true,
        self::MODE => self::MODE_BOTH,
        self::CALLBACK => null,
    );
    
    public function __construct($path, $options = null)
    {
        $this->_basePath = realpath($path);
        foreach ($options as $key => $value) {
            $this->_options[$key] = $value;
        }
        $this->rewind();
    }
    
    public function rewind()
    {
        $this->_dirs = array($this->_basePath);
        $this->_subdirs = null;
        $this->_dir = null;
        $this->_file = null;
        $this->_index = -1;
        
        $this->next();
    }
    
    public function key()
    {
        $this->current();
        return $this->_index;
    }
    
    public function current()
    {
        if ($this->_file === null) {
            $this->next();
        }
        return $this->_file !== null ? $this->_file : false;
    }
    
    public function valid()
    {
        if ($this->_file === null) {
            $this->next();
        }
        return $this->_file !== null;
    }
    
    public function next()
    {
        while (true) {
            if ($this->_dir !== null) {
                if ($file = array_shift($this->_handle)) {
                    if (!in_array($file, array('.','..'))) {
                        $this->_file = $this->_dir.DIRECTORY_SEPARATOR.$file;
                        
                        $isd = is_dir($this->_file);
                        
                        if ($this->_options[self::RECURSE] && $isd) {
                            array_push($this->_subdirs, $this->_file);
                        }
                        
                        if ( ($isd && ($this->_options[self::MODE] & self::MODE_DIRS)) ||
                            (!$isd && ($this->_options[self::MODE] & self::MODE_NONDIRS)) ) {
                            if ($this->accept()) {
                                $this->_index++;
                                break;
                            }
                        }
                    }
                } else {
                    array_splice($this->_dirs, 0, 0, $this->_subdirs);
                    $this->_dir = null;
                }
            } else if (count($this->_dirs) > 0) {
                $this->_dir = array_shift($this->_dirs);
                $this->_handle = scandir($this->_dir);
                $this->_subdirs = array();
            } else {
                $this->_dir = null;
                $this->_file = null;
                break;
            }
        }
    }
    
    public function accept()
    {
        if ($this->_options[self::CALLBACK] === null) {
            return true;
        } else {
            return call_user_func($this->_options[self::CALLBACK], $this);
        }
    }
}
?>