<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Filesystem
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

class Webtatic_Filesystem_Path {
    protected $_path;
    
    public function getPath() {
        return $this->_path;
    }
    
    public function __toString() {
        return $this->getPath();
    }
    
    public function __construct($path, $base = '.') {
        $this->_path = $this->_resolve($path, false, $base);
    }
    
    public function exists() {
        return file_exists($this->_path);
    }
    
    public function getRealPath() {
        return $this->_resolve($this->_path, true);
    }
    
    protected function _getWorkingDirectory() {
        return getcwd();
    }
    
    protected function _readLink($path) {
        return @readlink($path);
    }
    
    protected function _resolve($path, $followSymlinks = false, $base = '.') {
        if ($path[0] != '/') {
            if ($base[0] == '.') {
                $base = $this->_getWorkingDirectory();
            } else {
                $base = $this->_resolve($base, $followSymlinks);
            }
            $path = $base.'/'.$path;
        }
        $offset = 0;
        $parts = array();
        $relative = false;
        while (preg_match('#^/([^/]*)#', $path, $match)) {
            switch ($match[1]) {
                case '.':
                case '':
                    break;
                case '..':
                    array_pop($parts);
                    break;
                default:
                    array_push($parts, $match[1]);
            }
            $path = substr($path, strlen($match[0]));
        }
        if ($followSymlinks) {
            $path = '';
            while ($part = array_shift($parts)) {
                if ($symlink = $this->_readLink($path.'/'.$part)) {
                    $path = $this->_resolve($symlink, true, $filename);
                } else {
                    $path .= '/' . $part;
                }
            }
            if ($path == '') {
                $path = '/';
            }
            return $path;
        }
        
        return '/'.join('/', $parts);
    }
    
    public static function resolve($path, $followSymlinks = false, $base = '.') {
        $file = new self($path, $base);
        if ($followSymlinks) {
            return $file->getRealPath();
        } else {
            return $file->getPath();
        }
    }
}
?>