<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Filter
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Zend/Filter/Interface.php';

class Webtatic_Filter_Url implements Zend_Filter_Interface
{
    protected $_defaultScheme;
    protected $_defaultDomain;
    
    /**
     * 'scheme', 'host', 'user', 'pass', 'path', 'query', 'fragment',
     *
     * @var array
     */
    protected $_preserve;
    
    protected $_defaultPreserve = array(
        
    );
    
    public function __construct($defaultScheme = 'http', $defaultBase = null, array $preserve = null)
    {
        $this->_defaultScheme = $defaultScheme;
        $this->_defaultBase = $defaultBase;
        $this->_preserve = $preserve;
    }
    
    public function filter($value)
    {
        if ($value[0] == '/' && $this->_defaultBase !== null) {
            return rtrim($this->_defaultBase, '/') . $value;
        } elseif (!preg_match('/^[a-z]+:\/\//', $value) && $this->_defaultScheme !== null) {
            $value = $this->_defaultScheme . '://' . $value;
        }
        
        $url = parse_url($value);
        
        if ($url['path'] == '') {
            $url['path'] = '/';  
        }
        
        if ($this->_preserve !== null) {
            $url = array_intersect_key($url, array_flip($this->_preserve));
        }
        
        $value = '';
        
        if ($url['scheme']) {
            $value = $url['scheme'] . '://';
        }
        
        if ($url['user'] && $url['pass']) {
            $value .= $url['user'] . ':' . $url['pass'] . '@';
        } else if ($url['user']) {
            $value .= $url['user'] . '@';
        }
        
        if ($url['host']) {
            $value .= $url['host'];
        }
        
        if ($url['path']) {
            $value .= $url['path'];
        }
        
        if ($url['query']) {
            $value .= '?' . $url['query'];
        }
        
        if ($url['fragment']) {
            $value .= '#' . $url['fragment'];
        }
        
        return $url['scheme'] . '://' . $url['host'] . $url['path'];
    }
}
?>