<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/FieldSet.php';

class Webtatic_Form_Action extends Webtatic_Form_FieldSet
{
    protected $_data;
    protected $_dataSet = true;

    protected $_processed = null;
    protected $_map;
    protected $_action;
    protected $_method = 'post';
    protected $_elements = array();

    public function __construct($name, $options = array())
    {
        $this->_name = $name;
        $this->setOptions($options);
    }

    public function setOption($name, $value)
    {
        $method = 'set'.ucfirst($name);
        if (method_exists($this, $method)) {
            return $this->$method($value);
        }
        
        $this->setField($name, $value);

        return $this;
    }

    /**
     * Enter description here...
     *
     * @param array $options
     * @return Webtatic_Form
     */
    public function setOptions($options)
    {
        foreach ($options as $key => $value) {
            $this->setOption($key, $value);
        }

        return $this;
    }

    public function getAction()
    {
        return $this->_action;
    }

    public function setAction($value)
    {
        $this->_action = $value;

        return $this;
    }

    public function getMethod()
    {
        return $this->_method;
    }

    public function setMethod($value)
    {
        $this->_action = $value;

        return $this;
    }

    public function loadClass($type, $name)
    {
        $name = ucfirst($name);

        if (!(include_once('Webtatic/Form/'.$type.'/'.$name.'.php'))) {
            throw new Exception('Invalid element file path \'Webtatic/Form/'.$type.'/'.$name.'.php\'');
        }

        $class = 'Webtatic_Form_' . $type . '_'.$name;

        return $class;
    }

    public function setStorage($name, $options = array())
    {
        if (is_object($name)) {
            if ($name instanceof Webtatic_Form_Storage_Abstract) {
                $this->_steps[] = $name;
            } else {
                throw new Webtatic_Form_Exception('Object of class "'.classname($name).'" does not implement "Webtatic_Form_Storage_Interface"');
            }
        } else {
            $class = $this->loadClass('Storage', $name);
            $this->_storage = new $class($this, $options);
        }

        return $this;
    }

    /**
     * Enter description here...
     *
     * @return Webtatic_Form_Storage_Session
     */
    public function getStorage()
    {
        if (!$this->_storage) {
            $this->setStorage('session');
        }

        return $this->_storage;
    }

    public function getData()
    {
        return $this->_data;
    }

    public function setData($value)
    {
        $this->_data = $value;
        $this->_dataSet = true;
        $this->loadData($this->_data, $this->_data == null);
        return $this;
    }
    
    public function process($partial = false)
    {
        return parent::process($partial);
    }

    public function render($view)
    {
        $html = '<form name="'.$view->escape($this->getName()).'"
            action="'.$view->escape($this->getAction()).'"
            method="'.$view->escape($this->getMethod()).'">';
        $html .= parent::render($view);

        $html .= '</form>';
        return $html;
    }

    public function getElement($name)
    {
        return $this->_elements[$name];
    }

    public function setElement($name, $value)
    {
        $this->_elements[$name] = $value;

        return $this;
    }

    public function getElements()
    {
        return $this->_elements;
    }

    public function setElements($options)
    {
        foreach ($options as $name => $value) {
            $this->setElement($name, $value);
        }
        return $this;
    }
}
?>