<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/Element/Abstract.php';

class Webtatic_Form_Element_Date extends Webtatic_Form_Element_Abstract
{
    
    protected $_defaultFormat = array('Y-m-d' => 'Y-m-d');
    
    /**
     * Enter description here...
     *
     * @param Zend_View $view
     * @param Webtatic_Form_Field_Abstract $field
     * @param Webtatic_Form_Data $data
     * @return unknown
     */
    public function render($view, $field, $formats = null)
    {
        $attributes = array(
            'id' => $field->getId(),
        ) + $field->getAttributes();
        
        $rules = $field->getRules();
        
        foreach (array_shift($rules)->getChain() as $item) {
            if ($item['object'] instanceof Webtatic_Form_Validate_Range) {
                $range = $item['object'];
                break;
            }
        }
        
        if (!$range) {
            throw new Exception('No range validator found in field rules');
        }
        
        $min = $range->getOption('min');
        $max = $range->getOption('max');
        
        if (!$formats) {
            if ($field instanceof Webtatic_Form_Field_Date) {
                $formats = $field->getFormat();
            } else {
                $formats = $this->_defaultFormat;
            }
        }
        
        $precedence = array('s', 'i', 'H', 'd', 'm', 'Y');
        
        $defaults = $field->getDefault();
        
        foreach ($formats as $format => $display) {
            $items = preg_split('/[-:]/', $format);
            
            $p = null;
            foreach ($items as $item) {
                $pn = array_search($item, $precedence);
                if ($p == null || $p > $pn) {
                    $p = $pn;
                }
            }
            switch ($precedence[$p]) {
                case 'Y':
                    $increment = '+1 year';
                    break;
                case 'm':
                    $min = mktime(0,0,0, date('m', $min), 1, date('Y', $min));
                    $increment = '+1 month';
                    break;
                case 'd':
                    $increment = '+1 day';
                    break;
                case 'h':
                    $increment = 3600;
                    break;
                case 'i':
                    $increment = 60;
                    break;
                case 's':
                    $increment = 1;
                    break;
            }
            
            $time = $min;
            $options = array();
            
            if ($defaults) {
                $options[''] = $defaults[$format];
            }
            
            
            while ($time <= $max) {
                $f = date($format, $time);
                if (array_key_exists($f, $options)) {
                    break;
                }
                
                $options[$f] = date($display, $time);
                if (is_string($increment)) {
                    $time = strtotime($increment, $time);
                } else {
                    $time += $increment;
                }
            }
            
            ksort($options);
            $value = $field->getRawValue();
            
            $attributes['name'] = $field->getFullName().'['.$format.']';
            
            $html .= '<select '.$this->_renderAttributes($view, $attributes).'>'."\n";
            foreach ($options as $key => $label) {
                $html .= '<option value="'.$view->escape($key).'"';
                
                if ($value !== null && $value[$format] == $key) {
                    $html .= ' selected="selected"';
                }
                $html .= '>'.$view->escape($label).'</option>'."\n";                
            }
            $html .= "\n</select>\n";
        }
        
        return $html;
        
    }

}
?>