<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/Field/Interface.php';

class Webtatic_Form_Field_Abstract implements Webtatic_Form_Field_Interface
{
    protected $_parent;
    protected $_name;
    protected $_rule;
    protected $_default;

    protected $_type;
    protected $_multiple = false;
    protected $_options = null;

    protected $_label = null;
    protected $_visible = true;
    protected $_attributes = array();
    protected $_defaultElement = 'text';
    protected $_element = null;
    protected $_required = false;
    protected $_readOnly = false;
    protected $_remember = true;
    protected $_helpTip = '';
    
    
    protected $_rawValue;
    protected $_rawValueSet = false;
    protected $_value;
    protected $_valid = null;
    protected $_errors = array();
    protected $_messages = array();
    
    public function __clone() {
        $this->_rule = clone $this->_rule;
    }

    /**
     * Enter description here...
     *
     * @param Webtatic_Form $input
     * @param string $name
     * @param array $options
     */
    public function __construct($name, $options = array(), $parent = null)
    {
        $this->_parent = $parent;
        $this->_name = $name;

        $ruleOptions = array(
            'fields' => array($name),
        );

        $this->_rule = new Webtatic_Form_Rule($parent, $name, $ruleOptions);
        $this->setOptions($options);
    }
    
    public function getParent()
    {
        return $this->_parent;
    }
    
    public function setParent($value)
    {
        $this->_parent = $value;
        $this->_rule->setFieldSet($value);
        
        return $this;
    }

    /**
     * @return Webtatic_Form
     */
    public function getFieldSet()
    {
        return $this->_fieldSet;
    }
    
    public function getRule()
    {
        return $this->_rule;
    }

    /**
     * Enter description here...
     *
     * @return Webtatic_Form_Rule
     */
    public function getRules()
    {
        return array($this->_rule->getName() => $this->_rule);
    }

    /**
     * Enter description here...
     *
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }
    
    public function getId()
    {
        $parentName = $this->getParent()->getId();
        if ($parentName != '') {
            return $parentName . '_' . $this->getName();
        } else {
            return $this->getName();
        }
    }
    
    public function getFullName()
    {
        $parentName = $this->getParent()->getFullName();
        if ($parentName != '') {
            return $parentName . '[' . $this->getName() . ']';
        } else {
            return $this->getName();
        }
    }
    
    /**
     *
     * @param Zend_View $view
     * @return string
     */
    
    public function getInfo()
    {
        $info = array(
            'valid' => $this->isValid(),
            'messages' => $this->getMessages(),
            'rawValue' => $this->getRawValue(),
            'default' => $this->getDefault(),
            'options' => $this->getValueOptions(),
        );
        return $info;
    }
    
    public function getRawValue($real = false)
    {
        if ($this->_rawValueSet && ($real || $this->getRemember())) {
            return $this->_rawValue;
        } else {
            return $this->_default;
        }
    }
    
    public function setRawValue($value)
    {
        $this->_rawValue = $value;
        $this->_rawValueSet = true;
        
        return $this;
    }
    
    public function getValue()
    {
        if (!$this->_valid) {
            $this->process(false);
        }
        return $this->_value;
    }
    
    public function setValue($value)
    {
        $this->_value = $value;
        
        return $this;
    }
    
    public function isValid($partial = false)
    {
        if ($this->_valid === null) {
            $this->process($partial);
        }
        return $this->_valid;
    }
    
    public function setValid($value)
    {
        $this->_valid = $value;
        
        return $this;
    }
    
    public function getErrors($partial = false)
    {
        if ($this->_valid === null) {
            $this->process($partial);
        }
        return $this->_errors;
    }
    
    public function setErrors($value)
    {
        $this->_errors = $value;
        
        return $this;
    }
    
    public function getMessages($partial = false)
    {
        if ($this->_valid === null) {
            $this->process($partial);
        }
        return $this->_messages;
    }
    
    public function setMessages($value)
    {
        $this->_messages = $value;
        
        return $this;
    }
    
    public function getOption($name)
    {
        $method = 'get'.ucfirst($name);
        if ($name != 'options' && method_exists($this, $method)) {
            return $this->$method();
        }
        
        switch ($name) {
            case 'validators':
            case 'chain':
                return $this->_rule->getOption($name);
            case 'options':
                return $this->getValueOptions();
            default:
                return $this->_attribute[$name];
        }
    }

    public function setOption($name, $value)
    {
        $method = 'set'.ucfirst($name);
        if ($name != 'options' && method_exists($this, $method)) {
            return $this->$method($value);
        }

        switch ($name) {
            case 'validators':
            case 'chain':
                $this->_rule->setOptions(array($name => $value));
                return $this;
            case 'options':
                return $this->setValueOptions($value);
            default:
                return $this->setAttribute($name, $value);
        }
    }

    /**
     * Enter description here...
     *
     * @param array $options
     * @return Webtatic_Form_Field
     */
    public function setOptions($options)
    {
        foreach ((array) $options as $key => $value) {
            $this->setOption($key, $value);
        }
        return $this;
    }
    
    public function __get($name)
    {
        return $this->getOption($name);
    }
    
    public function __set($name, $value)
    {
        return $this->setOption($name, $value);
    }

    public function getHelpTip()
    {
        return $this->_helpTip;
    }

    public function setHelpTip($value)
    {
        $this->_helpTip = $value;
    }

    public function getLabel()
    {
        if ($this->_label === null) {
            return $this->getName();
        } else {
            return $this->_label;
        }
    }

    public function setLabel($value)
    {
        $this->_label = $value;

        return $this;
    }

    public function getVisible()
    {
        return $this->_visible;
    }

    public function setVisible($value)
    {
        $this->_visible = $value;

        return $this;
    }

    public function getElement()
    {
        return $this->_element;
    }

    public function setElement($value)
    {
        $this->_element = $value;

        return $this;
    }

    public function getDefault()
    {
        return $this->_default;
    }

    public function setDefault($value)
    {
        $this->_default = $value;

        return $this;
    }

    public function getRequired()
    {
        return $this->_required;
    }

    public function setRequired($value)
    {
        $this->_required = $value;

        return $this;
    }
    
    public function getReadOnly()
    {
        return $this->_readOnly;
    }

    public function setReadOnly($value)
    {
        $this->_readOnly = $value;

        return $this;
    }
    
    public function getRemember()
    {
        return $this->_remember;
    }

    public function setRemember($value)
    {
        $this->_remember = $value;

        return $this;
    }    

    public function setAttribute($name, $value)
    {
        $this->_attributes[$name] = $value;
    }

    public function setAttributes($attributes)
    {
        foreach ($attributes as $key => $value) {
            $this->setAttribute($key, $value);
        }

        return $this;
    }

    public function getAttributes()
    {
        return $this->_attributes;
    }

    public function getAllowEmpty()
    {
        return $this->_allowEmpty;
    }

    public function setAllowEmpty($value)
    {
        $this->_allowEmpty = $value;

        return $this;
    }
    
    protected function _preProcess()
    {
        if ($this->getValueOptions()) {
            $this->_rule->addValidator(array('inArray',
                'options' => array_keys($this->getValueOptions()),
            ), true);
        }
        if ($this->getRequired()) {
            foreach ($this->_rule->getChain() as $item) {
                if ($item['object'] instanceof Webtatic_Form_Validate_NotEmpty) {
                    return;
                }
            }
            $this->_rule->addValidator('notEmpty', true);
        }
    }
    
    protected function _process($value, $partial = false)
    {
        if (in_array($value, array(null, '')) && !$this->getRequired()) {
            $this->setValid(true);
        } else {
            if (!$partial) {
                $args = $this->_rule->process(array($value), $this->getRequired());
                if ($this->_rule->isValid()) {
                    $this->setValid(true);
                    $value = $args['output'][0];
                } else {
                    $this->setValid(false);
                }
            }
            if ($this->_rule->hasRun()) {
                $this->setErrors($this->_rule->getErrors());
                $this->setMessages($this->_rule->getMessages($this->getLabel()));
            }
        }
        return $value;
    }

    public function process($partial = false)
    {
        $this->_preProcess();
        
        if ($this->getMultiple()) {
            $result = array();
            foreach ((array) $this->getRawValue(true) as $key => $value) {
                $result[$key] = $this->_process($value, $partial);
                if ($this->_valid === false) {
                    return;
                }
            }
            $this->setValue($result);
        } else {
            $this->setValue($this->_process($this->getRawValue(true), $partial));
        }
    }

    public function getMultiple()
    {
        return $this->_multiple;
    }

    public function setMultiple($value)
    {
        $this->_multiple = $value;

        return $this;
    }

    public function getValueOptions($filterOptions = null)
    {
        if ($filterOptions) {
            $filterOptions = (array) $filterOptions;
            $options = array();
            if (key($filterOptions) === 0) {
                foreach ( $filterOptions as $key => $value) {
                    if (array_key_exists($value, $this->_options)) {
                        $options[$value] = $this->_options[$value];
                    }
                }
            } else {
                foreach ( $filterOptions as $key => $value) {
                    if (array_key_exists($key, $this->_options)) {
                        $options[$key] = $value;
                    }
                }
            }
            return $options;
        } else {
            return $this->_options;
        }
    }

    public function setValueOptions($value)
    {
        $this->_defaultElement = 'select';
        $this->_options = $value;

        return $this;
    }

    public function getType()
    {
        return $this->_type;
    }

    public function setType($value)
    {
        $this->_type = $value;

        return $this;
    }
    
    public function reset()
    {
        $this->_rawValue = null;
        $this->_rawValueSet = false;
        $this->_value = null;
        $this->_errors = array();
        $this->_messages = array();
    }

    public function render($view, $element = null, $options = null)
    {
        if ($this->_valid === null) {
            $this->process(true);
        }
        if (!$element) {
            $element = $this->_element ? $this->_element :$this->_defaultElement;
        }

        $form = $this->getParent()->getForm();
        $class = $form->loadClass('Element', $element);

        $element = new $class($form);
        if ($options) {
            return $element->render($view, $this, $options);
        } else {
            return $element->render($view, $this);
        }
    }
    
    public function renderLabel($view, $label = null)
    {
        return $this->render($view, 'label', $label);
    }
}
