<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form.php';
require_once 'Webtatic/Form/Field/Interface.php';
require_once 'Webtatic/Form/Field.php';

class Webtatic_Form_Field_Choice extends Webtatic_Form_FieldSet
    implements Webtatic_Form_Field_Interface
{
    protected $_forms = array();
    
    public function __construct($name, $options = array(), $parent = null)
    {
        parent::__construct($name, array(), $parent);
        $this->setField('select', array('element' => 'radio', 'required' => true));
        $this->setOptions($options);
    }
    
    public function setOption($name, $value)
    {
        $select = $this->getField('select');
        
        $method = 'set'.ucfirst($name);
        if (method_exists($this, $method)) {
            return $this->$method($value);
        } elseif (method_exists($select, $method)) {
            $select->$method($value);
            return $this;
        } else {
            if ($value instanceof Webtatic_Form_FieldSet) {
                $form = $value;
            } else {
                $form = new Webtatic_Form_FieldSet($name, $value, $this->getParent());
            }
                
            $this->_forms[$name] = $form;
            
            $this->setField($name, $form);
            $select->setValueOptions($this->getValueOptions());
                
            return $this;
        }
    }
    
    public function setDefault($value)
    {
        $select = $this->getField('select');
        $select->setDefault($value);
        return $this;
    }
    
    public function getValueOptions()
    {
        $options = array();
        foreach ($this->_forms as $name => $form) {
            $options[$name] = $form->getLabel();
        }
        return $options;
    }
    
    public function loadData($data, $default = false)
    {
        parent::loadData($data, $default);
        
        foreach ((array) $this->_forms as $key => $field) {
                if ($field->getMultiple()) {
//                    if ($default) {
//                        $data[$key] = array_fill(0, $field->getMultiple(), null);
//                    }
//                    $d = array();
//                    foreach ((array) $data[$key] as $k => $v) {
//                        $df = new Webtatic_Form_DataSet($dataSet, $field, count($d));
//                        $d[] = $df;
//                        $field->loadData($df, $v, $default);
//                    }
//                    $dataSet->setData($d, $key);
                } else {
                    $field->loadData($data[$key], $default);
                }
        }
    }
    
    public function process($partial = false)
    {
        $valid = true;
        $errors = array();
        $messages = array();

        if ($this->testConditions()) {
            $select = $this->getField('select');
            
            if (!$this->select->isValid($partial)) {
                $valid = false;
            }
            $errors = array_merge($errors, $this->select->getErrors($partial));
            $messages = array_merge($messages, $this->select->getMessages($partial));
            if ($valid) {
                $form = $this->_forms[$this->select->value];
                //$data = $dataSet->getData($this->select->getFilteredValue());
                if ($form->getMultiple()) {
//                    foreach ($data as $k => $v) {
//                        $errors = array_merge($errors, $v->getErrors($partial));
//                        $messages = array_merge($messages, $v->getMessages($partial));
//                        if (!$v->isValid($partial)){
//                            $valid = false;
//                        }
//                    }
                } else {
                    $errors = array_merge($errors, $form->getErrors($partial));
                    $messages = array_merge($messages, $form->getMessages($partial));
                    if (!$form->isValid($partial)) {
                        $valid = false;
                    }
                }
            }
        }
        $this->setValid($valid);
        $this->setErrors($errors);
        $this->setMessages($messages);
    }
    
    public function render($view, $data = null)
    {
        return $view->partial($this->getTemplate(), array(
            'field' => $this->getField('select'),
            'form' => $this,
            'data' => $data,
        ));
    }
}
?>