<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

class Webtatic_Form_Filter_Url
{
    protected $_defaultScheme = 'http';
    protected $_defaultBase = null;
    protected $_preserve = null;
    
    public function getDefaultScheme()
    {
        return $this->_defaultScheme;
    }
    
    public function setDefaultScheme($value)
    {
        $this->_defaultScheme = $value;
        
        return $this;
    }
    
    public function getDefaultBase()
    {
        return $this->_defaultBase;
    }
    
    public function setDefaultBase($value)
    {
        $this->_defaultBase = $value;
        
        return $this;
    }
    
    public function getPreserve()
    {
        return $this->_preserve;
    }
    
    public function setPreserve($value)
    {
        $this->_preserve = $value;
    }
    
    public function filter($value)
    {
        if ($value[0] == '/' && $this->_defaultBase !== null) {
            return rtrim($this->_defaultBase, '/') . $value;
        } elseif (!preg_match('/^[a-z]+:\/\//', $value) && $this->_defaultScheme !== null) {
            $value = $this->_defaultScheme . '://' . $value;
        }
        
        $url = parse_url($value);
        
        if ($url['path'] == '') {
            $url['path'] = '/';  
        }
        
        if ($this->_preserve !== null) {
            $url = array_intersect_key($url, array_flip($this->_preserve));
        }
        
        $value = '';
        
        if ($url['scheme']) {
            $value = $url['scheme'] . '://';
        }
        
        if ($url['user'] && $url['pass']) {
            $value .= $url['user'] . ':' . $url['pass'] . '@';
        } else if ($url['user']) {
            $value .= $url['user'] . '@';
        }
        
        if ($url['host']) {
            $value .= $url['host'];
        }
        
        if ($url['path']) {
            $value .= $url['path'];
        }
        
        if ($url['query']) {
            $value .= '?' . $url['query'];
        }
        
        if ($url['fragment']) {
            $value .= '#' . $url['fragment'];
        }
        
        return $url['scheme'] . '://' . $url['host'] . $url['path'];
    }
}
?>