<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

class Webtatic_Form_Rule
{
    protected $_name;
    protected $_fields;
    protected $_chain = array();

    protected $_valid = null;
    protected $_errors;
    protected $_messages;

    /**
     * @var Webtatic_Form_FieldSet
     */
    protected $_fieldSet;
    
    /**
     * Enter description here...
     *
     * @param string $name
     * @param array $options
     */
    public function __construct($fieldSet, $name, $options = array())
    {
        $this->_fieldSet = $fieldSet;
        $this->_name = $name;
        $this->setOptions($options);
    }

    public function setOption($name, $value)
    {
        $method = 'set'.ucfirst($name);
        if (method_exists($this, $method)) {
            return $this->$method($value);
        }
    }

    /**
     * Enter description here...
     *
     * @param array $options
     * @return Webtatic_Form_Rule
     */
    public function setOptions($options)
    {
        foreach ($options as $key => $value) {
            $this->setOption($key, $value);
        }

        return $this;
    }

    public function getName()
    {
        return $this->_name;
    }

    /**
     * Enter description here...
     *
     * @return array
     */
    public function getFields()
    {
        return $this->_fields;
    }

    public function setFields($value)
    {
        $this->_fields = (array) $value;

        return $this;
    }
    
    /**
     * @return Webtatic_Form_FieldSet
     */
    public function getFieldSet()
    {
        return $this->_fieldSet;
    }
    
    public function setFieldSet($value)
    {
        $this->_fieldSet = $value;
        
        return $this;
    }
    
    public function getForm()
    {
        return $this->getFieldSet()->getForm();
    }

    /**
     * Enter description here...
     *
     * @param string|Webtatic_Form_Validate_Abstract $name
     * @param array $options
     * @return Webtatic_Form_Rule
     */
    public function addValidator($options, $beginning = false)
    {
        if (is_object($options)) {
            if ($name instanceof Webtatic_Form_Validate_Abstract) {
                $this->_validators[] = $options;
                $item = array('type' => 'validator', 'object' => new $options, 'breakChainOnFailure' => true);
            } else {
                throw new Webtatic_Form_Exception('Object of class "'.classname($options).'" is not extended from "Webtatic_Form_Validate_Abstract"');
            }
        } else {
            $options = (array) $options;
            $name = array_shift($options);
            $class = Webtatic_Form::loadClass('Validate', $name);
            $item = array('type' => 'validator', 'object' => new $class($options), 'breakChainOnFailure' => $options['breakChainOnFailure'] ? $options['breakChainOnFailure'] : true);
        }
        
        if ($beginning) {
            array_unshift($this->_chain, $item);
        } else {
            $this->_chain[] = $item;
        }
        
        return $this;
    }

    /**
     * Enter description here...
     *
     * @param array $validators
     * @return Webtatic_Form_Rule
     */
    public function setValidators($validators)
    {
        foreach ((array) $validators as $options) {
            $this->addValidator($options);
        }

        return $this;
    }
    
    public function addFilter($options, $beginning = false)
    {
        if (is_object($options)) {
            if ($name instanceof Webtatic_Form_Filter_Abstract) {
                $this->_validators[] = $options;
            } else {
                throw new Webtatic_Form_Exception('Object of class "'.classname($options).'" is not extended from "Webtatic_Form_Filter_Abstract"');
            }
        } else {
            $options = (array) $options;
            $name = array_shift($options);
            $class = Webtatic_Form::loadClass('Filter', $name);
            $filter = array(
                'type' => 'filter',
                'object' => new $class(),
            );
            
            $filter['breakChainOnFailure'] = $options['breakChainOnFailure'] ? $options['breakChainOnFailure'] : true;
            $filter['filter'] = $options['filter'] ? $options['filter'] : 'output';
            unset($options['filter']);
            
            foreach ($options as $key => $value) {
                $filter['object']->{'set'.ucfirst($key)}($value);
            }
            
            if ($beginning) {
                array_unshift($this->_chain, $filter);
            } else {
                $this->_chain[] = $filter;
            }
        }
        return $this;
    }
    
    public function setFilters($filters)
    {
        foreach ((array) $filters as $options) {
            $this->addFilter($options);
        }
        
        return $this;
    }
    
    public function getChain()
    {
        return $this->_chain;
    }

    public function setChain($chain)
    {
        foreach ($chain as $item) {
            $type = array_shift($item);
            switch ($type) {
                case 'validator':
                    $this->addValidator($item);
                    break;
                case 'filterInput':
                    $item['filter'] = 'input';
                    $this->addFilter($item);
                    break;
                case 'filter':
                    $this->addFilter($item);
                    break;
            }
        }

        return $this;
    }

    public function process($arguments, $required)
    {
        $this->_valid = true;
        $this->_messages = array();
        $this->_errors = array();
        $result = array(
            'input' => $arguments,
            'output' => $arguments,
        );
        
        foreach ($this->_chain as $item) {
            switch ($item['type']) {
                case 'validator':
                    if (!call_user_func_array(
                        array($item['object'], 'isValid'), $arguments)) {
                        $this->_valid = false;
                        $this->_errors = array_merge($this->_errors, $item['object']->getErrors());
                        if (!isset($item['breakChainOnFailure']) || $item['breakChainOnFailure']) {
                            return;
                        }
                    }
                    $this->_errors = array_merge($this->_errors, $item['object']->getErrors());
                    break;
                case 'filter':
                    $result[$item['filter']] = call_user_func_array(
                        array($item['object'], 'filter'), $arguments);
                    if ($item['filter'] == 'input') {
                        $result['output'] = $result['input'];
                    }
                    
                    foreach ($result as $type => $args) {
                        if ($args === null) {
                            $result[$type] = array_fill(0, count($arguments), null);
                        } else if (!is_array($args)) {
                            $result[$type] = array($args);
                        }
                    }
                    
                    $arguments = $result['output'];
                    
                    if (method_exists($item['object'], 'getErrors')) {
                        $this->_errors = array_merge($this->_errors, $item['object']->getErrors());
                    }
                    
                    foreach ($arguments as $argument) {
                        if ($required || !in_array($argument, array(null, ''))) {
                            break 2;
                        }
                    }
                    
                    return $result;
            }
        }

        return $result;
    }
    
    public function hasRun()
    {
        return $this->_valid !== null;
    }

    public function isValid()
    {
        return $this->_valid;
    }

    public function getErrors()
    {
        return $this->_errors;
    }

    public function getMessages($label)
    {
        $messages = array();
        foreach ($this->_chain as $item) {
            if ($item['type'] == 'validator') {
                $messages = array_merge($messages,
                    $item['object']->getMessages(array('label' => $label)));
            }
        }
        return $messages;
    }
}
?>