<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

abstract class Webtatic_Form_Validate_Abstract
{
    
    protected $_messageTemplates = null;
    protected $_errors = array();
    protected $_options = array();
    
    public abstract function isValid($value);
    
    public function __construct($options = array())
    {
        $this->setOptions($options);
    }
    
    public function getOptions()
    {
        return $this->_options;
    }
    
    public function setOptions($options)
    {
        foreach ((array)$options as $key => $value) {
            if ($key != 'options' && method_exists($this, 'set'.$key)) {
                $this->{'set'.$key}($value);
            } else {
                $this->_options[$key] = $value;
            }
        }
        
        return $this;
    }
    
    public function getOption($name)
    {
        return $this->_options[$name];
    }

    public function getErrors()
    {
        return $this->_errors;
    }
    
    protected function _formatMessage($messageKey, $params)
    {
        $search = array();
        $replace = array();
        foreach ($params as $key => $value) {
            if (is_scalar($value)) {
                $search[] = '%'.$key.'%';
                $replace[] = (string) $value;
            }
        }
        return str_replace($search, $replace, $this->_messageTemplates[$messageKey]);
    }
    
    public function getMessages($additional = array())
    {
        $messages = array();
        foreach ($this->_errors as $messageKey => $params) {
            $messages[] = $this->_formatMessage($messageKey, $params + $additional);
        }
        return $messages;
    }
    
    public function setMessageTemplate($messageKey, $template)
    {
        $this->_messageTemplates[$messageKey] = $template;
    }
    
    public function getMessageTemplate($messageKey)
    {
        return $this->_messageTemplates[$messageKey];
    }
    
    public function getMessageTemplates()
    {
        return $this->_messageTemplates;
    }
    
    public function setMessageTemplates($templates)
    {
        foreach ($templates as $key => $value) {
            $this->setMessageTemplate($key, $value);
        }
        
        return $this;
    }
    
    protected function _error($messageKey = null, $value, $params = array())
    {
        $params['value'] = $value;
        $this->_errors[$messageKey] = $this->_options + $params;
    }
}
?>