<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/Validate/Abstract.php';

class Webtatic_Form_Validate_Identical extends Webtatic_Form_Validate_Abstract
{

    protected $_messageTemplates = array(
        'notIdenticalField' => '<b>%label%</b> does not match',
        'notIdenticalValue' => '<b>%label%</b> does not match',
        'identicalField' => '<b>%label%</b> does not match',
        'identicalValue' => '<b>%label%</b> does not match',
    );

    public function isValid($value)
    {
        $values = func_get_args();
        $value = array_shift($values);
        $strict = $this->getOption('strict') ? true : false;
        
        $not = $this->getOption('not') ? true : false;
        
        if ($not) {
            foreach ($values as $oValue) {
                if (($strict && $oValue === $value) || (!$strict && $oValue == $value)) {
                    $this->_error('identicalField', $value);
                    return false;
                }
            }
            
            $values = (array) $this->getOption('values');
            
            foreach ($values as $oValue) {
                if ( ($strict && $oValue === $value) || (!$strict && $oValue == $value) ) {
                    $this->_error('identicalValue', $value);
                    return false;
                }
            }
        } else {
            foreach ($values as $oValue) {
                if (($strict && $oValue != $value) || (!$strict && $oValue != $value)) {
                    $this->_error('notIdenticalField', $value);
                    return false;
                }
            }
            
            $values = (array) $this->getOption('values');
            
            foreach ($values as $oValue) {
                if ( ($strict && $oValue !== $value) || (!$strict && $oValue != $value) ) {
                    $this->_error('notIdenticalValue', $value);
                    return false;
                }
            }
        }
        return true;
    }
}
?>
