<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/Validate/Abstract.php';

class Webtatic_Form_Validate_PostCode extends Webtatic_Form_Validate_Abstract
{
    
    protected $_messageTemplates = array(
        'invalid' => 'Your <b>%label%</b> is not recognised',
    );
    
    function fixNumber($mistake)
    {
        switch ($mistake) {
            case 'O': return '0';
            case 'I':
            case 'L': return '1';  
            case 'Z': return '2';
            case 'S': return '5';
        }
        return $mistake;
    }

    public function isValid($postcode)
    {
        if ($postcode && !preg_match('/\D{3,}/', $postcode)) {
            if (preg_match('/[0-9][A-Z]{1,2}$/i', $postcode)) {
                $postcode = str_replace(' ', '', $postcode);
            }

            $postcode = strtoupper( preg_replace( '/[[:punct:]]+/', '', $postcode ) );

            if (preg_match('/([A-Z0-9]{2,4})(.*)$/', $postcode, $regs)) {
                if (preg_match('/[0-9][A-Z]+$/', $regs[1], $regs2) &&
                        !preg_match('/^[0-9]/', $regs[2]) && $regs[2]) {
                    $regs[2] = $regs2[0].$regs[2];
                    $regs[1] = str_replace($regs2[0], '', $regs[1]);
                }

                if (preg_match('/^[A-Z]/', $regs[2]) && (strlen($regs[2]) != 3)) {
                    $regs[2] = $regs[1]{strlen($regs[1])-1}.$regs[2];
                    $regs[1] = substr_replace($regs[1], '', strlen($regs[1])-1);
                }
                
                $regs[1]{2} = $this->fixNumber($regs[1]{2});
                $regs[1]{3} = $this->fixNumber($regs[1]{3});
                $regs[2]{0} = $this->fixNumber($regs[2]{0});

                if (!$regs[2]){
                    $this->_error('invalid', $postcode);
                    return false; // district postcode
                } else if ( preg_match('/([0-9][A-Z]{2})/', $regs[2]) ){
                    return true; // full postcode
                } else {
                    $this->_error('invalid', $postcode);
                    return false; // partial postcode
                }
            }
        }
        return false;
    }
}
?>