<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Form
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

require_once 'Webtatic/Form/Validate/Abstract.php';

class Webtatic_Form_Validate_Range extends Webtatic_Form_Validate_Abstract
{
    public $_break = true;
    
    public function __construct($options = array())
    {
        parent::__construct($options);
        
        $this->_messageTemplates = array(
            'minInclusive' => '<b>%label%</b> cannot be under %min%',
            'maxInclusive' => '<b>%label%</b> cannot exceed %max%',
            'minExclusive' => '<b>%label%</b> must be over %min%',
            'maxExclusive' => '<b>%label%</b> must be under %max%',
        );
    }
    
    public function isValid($value)
    {
        if (!isset($this->_options['inclusive']) || $this->_options['inclusive']) {
            if (isset($this->_options['min']) && ($value < $this->_options['min'])) {
                $this->_error('minInclusive', $value);
            }
            
            if (isset($this->_options['max']) && ($value > $this->_options['max'])) {
                $this->_error('maxInclusive', $value);
            }
        } else {
            if (isset($this->_options['min']) && ($value <= $this->_options['min'])) {
                $this->_error('minExclusive', $value);
            }
            
            if (isset($this->_options['max']) && ($value >= $this->_options['max'])) {
                $this->_error('maxExclusive', $value);
            }
        }
        
        if (count($this->_errors) > 0) {
            return false;
        }
        return true;
    }
}
?>