<?php
/**
 * This file is part of Webtatic Framework.
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.webtatic.com/license/new-bsd
 *
 * @category   Webtatic
 * @package    Webtatic_Parser
 * @copyright  Copyright (c) 2007-2009 Andrew Thompson. (http://www.webtatic.com)
 * @license    http://www.webtatic.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

abstract class Webtatic_Parser_Abstract
{
    protected $_patterns;
    protected $_defaultState = null;
    protected $_state = null;
    protected $_missCallback = null;
    protected $_config;
    
    protected $_compiledPatterns = null;
    protected $_compiledRegExps = null;
    
    protected $_s = array();
    
    public function setState($value)
    {
        $this->_state = $value;
    }
    
    public function __construct($config = array())
    {
        $this->_config = $config;
    }
    
    protected function simplifyPatterns($state, $patterns, $append = null)
    {
        $ps = array();
        foreach ($patterns as $pattern) {
            if ($append !== null) {
                $pattern = array_merge($pattern, $append);
            }
            $i = 0;
            
            while (list($key, $p) = each($pattern)) {
                if ($p[0] == '#') {
                    $p1 = substr($p, 1);
                    $f = $this->simplifyPatterns($state, $this->_patterns[$p1], array_slice($pattern, 1));
                    $ps = array_merge($ps, $f);
                } else {
                    if (list(, $p2) = each($pattern)) {
                        if ($p2[0] != '#' || each($pattern)) {
                            $s = $state . '-' . $i++;
                            $ps[] = array($key, $p, $s);
                            $this->_patterns[$s] = array(array_slice($pattern, 1));
                            $this->_compiledPatterns[$s] = 
                                $this->simplifyPatterns($s, $this->_patterns[$s]);
                        } else {
                            $ps[] = array($key, $p, substr($p2, 1));
                        }
                    } else {
                        $ps[] = array($key, $p);
                    }
                }
                break;
            }
        }
        return $ps;
    }
    
    protected function compilePatterns()
    {
        $this->_compiledPatterns = array();
        foreach ($this->_patterns as $state => $patterns) {
            $this->_compiledPatterns[$state] = $this->simplifyPatterns($state, $patterns, null);
        }
        foreach ($this->_compiledPatterns as $state => $patterns) {
            $regExp = array();
            $i = 0;
            foreach ($patterns as $pattern) {
                $regExp[] = '(?P<match_'.$pattern[0].'_'.$i.'>'.$pattern[1].')';
                $i++;
            }
            if ($this->_missCallback == null) {
                $start = '\G';
            } else {
                $start = '';
                $options = '';
            }
            $this->_compiledRegExps[$state] =
                '#' . $start . join('|', $regExp) . '#s';
        }
    }
    
    protected function formatPassthrough($match)
    {
        return $match[0][0];
    }
    
    protected function formatSkip($match)
    {
        return '';
    }
    
    protected function _parse($content, $state = null)
    {
        if ($state === null) {
            $state = $this->_defaultState;
        }
        
        $oldState = $this->_state;
        $this->_state = $state;
        
        $this->compilePatterns();
        $output = '';
        $offset = 0;
        
        $this->_s = array($state);
        while (preg_match($this->_compiledRegExps[$state], $content, $match, PREG_OFFSET_CAPTURE, $offset)) {
            if ($this->_missCallback !== null) {
                $miss = array(
                    substr($content, $offset, $match[0][1] - $offset),
                    $offset,
                );
                if (strlen($miss[0]) > 0) {
                    $output .= $this->{'format'.$this->_missCallback}(array($miss));
                }
            }
            $i = 0;
            
            $md = array_shift($match);
            foreach ($match as $key => $value) {
                if ($value[1] >= 0) {
                    $m = array_slice($match, $i+1);
                    $key = explode('_', $key);
                    $pattern = $this->_compiledPatterns[$state][$key[2]];
                    
                    $action = 'format'.$key[1];
                    $output .= $this->$action($m);
                    
                    if (isset($pattern[2])) {
                        if ($state != $pattern[2]) {
                            $this->_s[] = $pattern[2];
                            $state = $pattern[2];
                        }
                    } else {
                        array_pop($this->_s);
                        $state = $this->_s[count($this->_s)-1];
                    }
                    $this->_state = $state;
                    break;
                }
                $i++;
            }
            
            $offset = $md[1] + strlen($md[0]);
        }
        
        if ($this->_missCallback !== null) {
            $miss = array(
                    substr($content, $offset),
                    0,
                );
            
            if (strlen($miss[0]) > 0) {
                $output .= $this->{'format'.$this->_missCallback}(array($miss));
            }
        }
        $this->_state = $oldState;
        return $output;
    }
}
?>